import os
import pprint
import gdcm

from .image import Image


class DataElement(object):
    """ Object representation of a Data Element """

    def __init__(self, swig_element, name, value):
        """ Executed on creation of Data Element

        :param swig_element: GDCMs DataElement object
        :param name: Name of DICOM Data Element
        :param value: Value of DICOM Data Element """
        self._swig_element = swig_element

        self.name = name
        self.value = value
        self.VR = str(swig_element.GetVR()).strip()
        self.VL = str(swig_element.GetVL()).strip()

        tg = swig_element.GetTag()
        self.tag = {
            "group": hex(int(tg.GetGroup())),
            "element": hex(int(tg.GetElement())),
            "str": str(swig_element.GetTag()).strip(),
        }


class Dicom(object):
    """ Primary class that loads the DICOM file into
    memory and has properties that allows for reading
    the DICOM elements, extracting images

    :param fname: Location and filename of DICOM file.
    """
    def __init__(self, fname):
        self.fname = fname
        self.file_name, self.file_extension = os.path.splitext(fname)

        reader = gdcm.Reader()
        reader.SetFileName(fname)
        if not reader.Read():
            raise Exception("Not a valid DICOM file")

        file_mem = reader.GetFile()
        self._header = file_mem.GetHeader()
        self._dataset = file_mem.GetDataSet()
        self._str_filter = gdcm.StringFilter()
        self._str_filter.SetFile(file_mem)

        self._pretty = pprint.PrettyPrinter(indent=4)

    def image(self):
        """ Read the loaded DICOM image data """
        return Image(self.fname)

    def read(self, pretty=False):
        """ Returns array of dictionaries containing
        all the data elements in the DICOM file.
        """
        def ds(data_element):
            value = self._str_filter.ToStringPair(data_element.GetTag())
            if value[1]:
                return DataElement(data_element, value[0].strip(), value[1].strip())

        results = [data for data in self.walk(ds) if data is not None]
        if pretty:
            return self._pretty.pprint(results)
        else:
            return results

    def walk(self, fn):
        """ Loops through all data elements and
        allows a function to interact with each data element.  Uses
        a generator to improve iteration.

        :param fn: Function that interacts with each DICOM element """
        if not hasattr(fn, "__call__"):
            raise Exception("""walk_dataset requires a
                function as its parameter""")

        dataset = self._dataset
        iterator = dataset.GetDES().begin()
        while (not iterator.equal(dataset.GetDES().end())):
            data_element = iterator.next()
            yield fn(data_element)

        header = self._header
        iterator = header.GetDES().begin()
        while (not iterator.equal(header.GetDES().end())):
            data_element = iterator.next()
            yield fn(data_element)

    def find(self, group=None, element=None, name=None, VR=None, pretty=False):
        """ Searches for data elements in the DICOM file given
        the filters supplied to this method.

        :param group: Hex decimal for the group of a DICOM element e.g. 0x002
        :param element: Hex decimal for the element value of a
        DICOM element e.g. 0x0010
        :param name: Name of the DICOM element, e.g. "Modality"
        :param VR: Value Representation of the DICOM element, e.g. "PN"
        """
        results = self.read()

        if name is not None:
            def find_name(data_element):
                if data_element.name.lower() == name.lower():
                    return True
                else:
                    return False
            return filter(find_name, results)

        if group is not None:
            def find_group(data_element):
                if (data_element.tag['group'] == group
                    or int(data_element.tag['group'], 16) == group):
                        return True
                else:
                    return False
            results = filter(find_group, results)

        if element is not None:
            def find_element(data_element):
                if (data_element.tag['element'] == element
                    or int(data_element.tag['element'], 16) == element):
                        return True
                else:
                    return False
            results = filter(find_element, results)

        if VR is not None:
            def find_VR(data_element):
                if data_element.VR.lower() == VR.lower():
                    return True
                else:
                    return False
            results = filter(find_VR, results)

        if pretty:
            return self._pretty.pprint(results)
        else:
            return results


from __future__ import division
import os.path
import sys
import numpy
import gdcm

class Image(object):
    """ This class attempts to extract an image
    out of the DICOM file.

    :param fname: Location and filename of DICOM file.
    """
    def __init__(self, fname):
        self.fname = fname

    def numpy(self):
        """ Grabs image data and converts it to a numpy array """
        # load GDCM's image reading functionality
        image_reader = gdcm.ImageReader()
        image_reader.SetFileName(self.fname)
        if not image_reader.Read():
            raise Exception("Could not read DICOM image")
        pixel_array = self._gdcm_to_numpy(image_reader.GetImage())
        return pixel_array

    def _gdcm_to_numpy(self, image):
        """ Converts a GDCM image to a numpy array.

        :param image: GDCM.ImageReader.GetImage()
        """
        gdcm_typemap = {
            gdcm.PixelFormat.INT8:     numpy.int8,
            gdcm.PixelFormat.UINT8:    numpy.uint8,
            gdcm.PixelFormat.UINT16:   numpy.uint16,
            gdcm.PixelFormat.INT16:    numpy.int16,
            gdcm.PixelFormat.UINT32:   numpy.uint32,
            gdcm.PixelFormat.INT32:    numpy.int32,
            gdcm.PixelFormat.FLOAT32:  numpy.float32,
            gdcm.PixelFormat.FLOAT64:  numpy.float64
        }
        pixel_format = image.GetPixelFormat().GetScalarType()
        if pixel_format in gdcm_typemap:
            self.data_type = gdcm_typemap[pixel_format]
        else:
            raise KeyError(''.join(pixel_format, \
                " is not a supported pixel format"))

        #dimension = image.GetDimension(0), image.GetDimension(1)
        self.dimensions = image.GetDimension(1), image.GetDimension(0)
        gdcm_array = image.GetBuffer()

        # GDCM returns char* as type str. This converts it to type bytes
        if sys.version_info >= (3, 0):
            gdcm_array = gdcm_array.encode(sys.getfilesystemencoding(), "surrogateescape")

        # use float for accurate scaling
        result = numpy.frombuffer(gdcm_array,
            dtype=self.data_type).astype(float)
        result.shape = self.dimensions
        return result

    def save_as_plt(self, fname, pixel_array=None, vmin=None, vmax=None,
        cmap=None, format=None, origin=None):
        """ This method saves the image from a numpy array using matplotlib

        :param fname: Location and name of the image file to be saved.
        :param pixel_array: Numpy pixel array, i.e. ``numpy()`` return value
        :param vmin: matplotlib vmin
        :param vmax: matplotlib vmax
        :param cmap: matplotlib color map
        :param format: matplotlib format
        :param origin: matplotlib origin

        This method will return True if successful
        """
        from matplotlib.backends.backend_agg \
        import FigureCanvasAgg as FigureCanvas
        from matplotlib.figure import Figure
        from pylab import cm

        if pixel_array is None:
            pixel_array = self.numpy()

        if cmap is None:
            cmap = cm.bone
        fig = Figure(figsize=pixel_array.shape[::-1], dpi=1, frameon=False)
        canvas = FigureCanvas(fig)
        fig.figimage(pixel_array, cmap=cmap, vmin=vmin,
            vmax=vmax, origin=origin)
        fig.savefig(fname, dpi=1, format=format)
        return True

    def save_as_pil(self, fname, pixel_array=None):
        """  This method saves the image from a
        numpy array using Pillow (PIL fork)

        :param fname: Location and name of the image file to be saved.
        :param pixel_array: Numpy pixel array, i.e. ``numpy()`` return value

        This method will return True if successful
        """
        if pixel_array is None:
            pixel_array = self.numpy()

        from PIL import Image as pillow
        pil_image = pillow.fromarray(pixel_array.astype('uint8'))
        pil_image.save(fname)
        return True



import subprocess

from .base import Dicom

def load(fname):
    """ Imports DICOM file into memory,
    returns a Dicom object.

    :param fname: Location and filename of DICOM file.
    """
    return Dicom(fname)

def validate(fname):
    """ This function uses dciodvfy to generate
    a list of warnings and errors discovered within
    the DICOM file.

    :param fname: Location and filename of DICOM file.
    """
    def _process():
        popen = subprocess.Popen(["dciodvfy", fname],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
        for line in iter(popen.stderr.readline, ""):
            yield line.replace("\n", "")

    def _determine(line):
        try:
            return "warnings", line[line.rindex("Warning - ")+10:]
        except ValueError:
            try:
                return "errors", line[line.index("Error - ")+8:]
            except ValueError:
                return None, None

    validation = {
        "errors": [],
        "warnings": []
    }

    for line in _process():
        kind, message = _determine(line)
        if kind in validation:
            validation[kind].append(message)
    return validation

def lookup_VR(VR=None, description=None):
    """ Value Representation (VR) <-> Description lookup.

    :param VR: Takes the VR and returns its description
    :param description: Take the description of a VR and returns the VR
    """
    value_repr = {
        "AE": "Application Entity",
        "AS": "Age String",
        "AT": "Attribute Tag",
        "CS": "Code String",
        "DA": "Date",
        "DS": "Decimal String",
        "DT": "Date/Time",
        "FL": "Floating Point Single (4 bytes)",
        "FD": "Floating Point Double (8 bytes)",
        "IS": "Integer String",
        "LO": "Long String",
        "LT": "Long Text",
        "OB": "Other Byte",
        "OF": "Other Float",
        "OW": "Other Word",
        "PN": "Person Name",
        "SH": "Short String",
        "SL": "Signed Long",
        "SQ": "Sequence of Items",
        "SS": "Signed Short",
        "ST": "Short Text",
        "TM": "Time",
        "UI": "Unique Identifier",
        "UL": "Unsigned Long",
        "UN": "Unknown",
        "US": "Unsigned Short",
        "UT": "Unlimited Text"
    }
    if VR:
        if VR in value_repr:
            return value_repr[VR]
        else:
            raise Exception("VR not found in map")
    elif description:
        for key, value in value_repr.iteritems():
            if description == value:
                return key
        raise Exception("Description not found in map")
    else:
        raise Exception("Either VR or description required to map VR")

def lookup_transfer_syntax(UID=None, description=None):
    """ Transfer Syntax UID <-> Description lookup.

    :param UID: Transfer Syntax UID, returns description
    :param description: Take the description of a transfer syntax and return its UID
    """

    transfer_syntax = {
        "1.2.840.10008.1.2": "Implicit VR Endian: Default Transfer Syntax for DICOM",
        "1.2.840.10008.1.2.1": "Explicit VR Little Endian",
        "1.2.840.10008.1.2.1.99": "Deflated Explicit VR Big Endian",
        "1.2.840.10008.1.2.2": "Explicit VR Big Endian",
        "1.2.840.10008.1.2.4.50": "JPEG Baseline (Process 1): Default Transfer Syntax for Lossy JPEG 8-bit Image Compression",
        "1.2.840.10008.1.2.4.51": "JPEG Baseline (Processes 2 & 4): Default Transfer Syntax for Lossy JPEG 12-bit Image Compression (Process 4 only)",
        "1.2.840.10008.1.2.4.57": "JPEG Lossless, Nonhierarchical (Processes 14)",
        "1.2.840.10008.1.2.4.70": "JPEG Lossless, Nonhierarchical, First-Order Prediction (Processes 14 [Selection Value 1])",
        "1.2.840.10008.1.2.4.80": "JPEG-LS Lossless Image Compression",
        "1.2.840.10008.1.2.4.81": "JPEG-LS Lossy (Near- Lossless) Image Compression",
        "1.2.840.10008.1.2.4.90": "JPEG 2000 Image Compression (Lossless Only)",
        "1.2.840.10008.1.2.4.91": "JPEG 2000 Image Compression",
        "1.2.840.10008.1.2.4.92": "JPEG 2000 Part 2 Multicomponent Image Compression (Lossless Only)",
        "1.2.840.10008.1.2.4.93": "JPEG 2000 Part 2 Multicomponent Image Compression",
        "1.2.840.10008.1.2.4.94": "JPIP Referenced",
        "1.2.840.10008.1.2.4.95": "JPIP Referenced Deflate",
        "1.2.840.10008.1.2.5": "RLE Lossless",
        "1.2.840.10008.1.2.6.1": "RFC 2557 MIME Encapsulation",
        "1.2.840.10008.1.2.4.100": "MPEG2 Main Profile Main Level",
        "1.2.840.10008.1.2.4.102": "MPEG-4 AVC/H.264 High Profile / Level 4.1",
        "1.2.840.10008.1.2.4.103": "MPEG-4 AVC/H.264 BD-compatible High Profile / Level 4.1"
    }
    if UID:
        if UID in transfer_syntax:
            return transfer_syntax[UID]
        else:
            raise KeyError("Transfer Syntax UID not found in map")
    elif description:
        for key, value in transfer_syntax.iteritems():
            if description == value:
                return key
        raise ValueError("Description not found in map")
    else:
        raise Exception("Either Transfer syntax UID or description required")


import os
import unittest
import numpy
import gdcm

import mudicom


class TestImage(unittest.TestCase):

	def setUp(self):
		nose_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "dicoms")
		self.fnames = (os.path.join(nose_dir, "ex1.dcm"),
			os.path.join(nose_dir, "ex2.dcm"),)

	def test_get_numpy(self):
		for fname in self.fnames:
			mu = mudicom.load(fname)
			img = mu.image()
			gnp = img.numpy()
			self.assertIsInstance(gnp, numpy.ndarray)
			self.assertEqual(gnp.dtype, numpy.float)



import os.path
import unittest
import gdcm

import mudicom


class TestRead(unittest.TestCase):

    def setUp(self):
        nose_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "dicoms")
        self.fnames = (os.path.join(nose_dir, "ex1.dcm"),
            os.path.join(nose_dir, "ex2.dcm"),)

    def test_get_dataset(self):
        for fname in self.fnames:
            print fname
            mu = mudicom.load(fname)
            ds = mu.read()
            self.assertIsInstance(ds, list)
            self.assertIsNot(len(ds), 0)

            for element in ds:
                self.assertTrue(hasattr(element, "name"))
                self.assertTrue(hasattr(element, "value"))
                self.assertTrue(hasattr(element, "VR"))
                self.assertTrue(hasattr(element, "VL"))
                self.assertTrue(hasattr(element, "tag"))
                self.assertTrue("group" in element.tag)
                self.assertTrue("element" in element.tag)
                self.assertTrue("str" in element.tag)

    def test_map_VR(self):
        self.assertEqual(mudicom.lookup_VR(VR="AE"), "Application Entity")
        self.assertEqual(mudicom.lookup_VR("OB"), "Other Byte")
        self.assertEqual(mudicom.lookup_VR("US"), "Unsigned Short")
        self.assertEqual(mudicom.lookup_VR(description="Time"), "TM")
        self.assertEqual(mudicom.lookup_VR(description="Unknown"), "UN")
        self.assertEqual(mudicom.lookup_VR(description="Sequence of Items"), "SQ")
        self.assertRaises(Exception, mudicom.lookup_VR, "ZZ")
        self.assertRaises(Exception, mudicom.lookup_VR, None, "Hi there")


    def test_walk_dataset(self):
        for fname in self.fnames:
            mu = mudicom.load(fname)
            #reader = mu.read()

            tags = list(mu.walk(lambda ds: ds.GetTag()))
            self.assertIsInstance(tags, list)
            self.assertIsNot(len(tags), 0)
            for tag in tags:
                self.assertIsInstance(tag, gdcm.Tag)

            tags = list(mu.walk(lambda ds: None))
            self.assertIsInstance(tags, list)
            for tag in tags:
                self.assertEqual(tag, None)

            tags = list(mu.walk(lambda ds: False))
            self.assertIsInstance(tags, list)
            for tag in tags:
                self.assertFalse(tag)

            tags = list(mu.walk(lambda ds: True))
            self.assertIsInstance(tags, list)
            for tag in tags:
                self.assertTrue(tag)

    def test_get_element(self):
        for fname in self.fnames:
            mu = mudicom.load(fname)
            #reader = mu.read()
            self.assertIsInstance(mu.find(0x004, 0x1220), list)
            self.assertIsInstance(mu.find(name="Modality"), list)
            self.assertIsInstance(mu.find(VR="PN"), list)
